<?php

namespace Drupal\default_widget\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure default_widget settings for this site.
 */
class DefaultWidgetSettingsForm extends ConfigFormBase {

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManager
   */
  protected $fieldTypePluginManager;

  /**
   * The widget plugin manager.
   *
   * @var \Drupal\Core\Field\WidgetPluginManager
   */
  protected $widgetPluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->fieldTypePluginManager = $container->get('plugin.manager.field.field_type');
    $instance->widgetPluginManager = $container->get('plugin.manager.field.widget');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'default_widget_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['default_widget.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $field_types = $this->config('default_widget.settings')->get();

    foreach ($field_types as $field_type => $config) {
      $field_type_definition = $this->fieldTypePluginManager->getDefinition($field_type);

      // Create the category grouping on the form if it doesn't exist already.
      $category = (string) $field_type_definition['category'];
      if (!isset($form[$category])) {
        $form[$category] = [
          '#type' => 'fieldset',
          '#title' => $field_type_definition['category'],
        ];
      }

      $form[$category][$field_type] = [
        '#type' => 'select',
        '#title' => $field_type_definition['label'],
        '#options' => $this->widgetPluginManager->getOptions($field_type),
        '#default_value' => $config['default_widget'],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('default_widget.settings');

    $field_types = $config->get();
    foreach ($field_types as $field_type => $definition) {
      $config->set("$field_type.default_widget", $form_state->getValue($field_type));
    }
    $config->save();

    parent::submitForm($form, $form_state);
  }

}
